class ImageConfig: ...
class StatsContext: ...

def gaussian_2d(width, height, h, k, a, b): ...
def gaussian_1d(width, height, h, k, ystd, xwidth): ...
def scaling2pix(x, y, bounds): ...
def coords2pix(x, y, bounds): ...
def write_image(data, filename, config, context, axis_limits) -> None: ...
def dress_image(img, bounds): ...
def points2img(fname) -> None: ...
