SCALE_RADIUS: float
ADD_RADIUS: float
MEAN: int
N: int

def data(dist): ...
def plot(data, title: str = ...) -> None: ...
def smear(data): ...
def generate_examples(visualize: bool = ...) -> None: ...
