from _typeshed import Incomplete

def file_ok(fname) -> None: ...
def filenames2stats1d(filenames): ...
def filenames2stats2d(filenames): ...
def regex2filenames(regex, dir: str = ...): ...
def regex2stats1d(regex, dir: str = ...): ...
def regex2stats2d(regex, dir: str = ...): ...
def file2filenames(filename): ...
def file2stats1d(filename): ...
def file2stats2d(filename): ...
def get_unique_tmpdir(name: str = ...): ...
def array2stats1d(*args): ...
def array2stats2d(*args): ...
def compare(*args, npeaks: Incomplete | None = ...): ...
